@@ -31,7 +31,7 @@ package jersey.repackaged.org.objectweb.asm;
 
 /**
  * Information about the input and output stack map frames of a basic block.
- * 
+ *
  * @author Eric Bruneton
  */
 final class Frame {
@@ -46,14 +46,14 @@ final class Frame {
      * of the first basic block (which is computed from the method descriptor),
      * and by using the previously computed output frames to compute the input
      * state of the other blocks.
-     * 
+     *
      * All output and input frames are stored as arrays of integers. Reference
      * and array types are represented by an index into a type table (which is
      * not the same as the constant pool of the class, in order to avoid adding
      * unnecessary constants in the pool - not all computed frames will end up
      * being stored in the stack map table). This allows very fast type
      * comparisons.
-     * 
+     *
      * Output stack map frames are computed relatively to the input frame of the
      * basic block, which is not yet known when output frames are computed. It
      * is therefore necessary to be able to represent abstract types such as
@@ -61,7 +61,7 @@ final class Frame {
      * position x from the top of the input frame stack" or even "the type at
      * position x in the input frame, with y more (or less) array dimensions".
      * This explains the rather complicated type format used in output frames.
-     * 
+     *
      * This format is the following: DIM KIND VALUE (4, 4 and 24 bits). DIM is a
      * signed number of array dimensions (from -8 to 7). KIND is either BASE,
      * LOCAL or STACK. BASE is used for types that are not relative to the input
@@ -70,16 +70,16 @@ final class Frame {
      * stack types. VALUE depends on KIND. For LOCAL types, it is an index in
      * the input local variable types. For STACK types, it is a position
      * relatively to the top of input frame stack. For BASE types, it is either
-     * one of the constants defined in FrameVisitor, or for OBJECT and
-     * UNINITIALIZED types, a tag and an index in the type table.
-     * 
+     * one of the constants defined below, or for OBJECT and UNINITIALIZED
+     * types, a tag and an index in the type table.
+     *
      * Output frames can contain types of any kind and with a positive or
      * negative dimension (and even unassigned types, represented by 0 - which
      * does not correspond to any valid type value). Input frames can only
      * contain BASE types of positive or null dimension. In all cases the type
      * table contains only internal type names (array type descriptors are
      * forbidden - dimensions must be represented through the DIM field).
-     * 
+     *
      * The LONG and DOUBLE types are always represented by using two slots (LONG
      * + TOP or DOUBLE + TOP), for local variable types as well as in the
      * operand stack. This is necessary to be able to simulate DUPx_y
@@ -107,7 +107,7 @@ final class Frame {
 
     /**
      * Mask to get the kind of a frame type.
-     * 
+     *
      * @see #BASE
      * @see #LOCAL
      * @see #STACK
@@ -489,10 +489,10 @@ final class Frame {
     /**
      * Relative size of the output stack. The exact semantics of this field
      * depends on the algorithm that is used.
-     * 
+     *
      * When only the maximum stack size is computed, this field is the size of
      * the output stack relatively to the top of the input stack.
-     * 
+     *
      * When the stack map frames are completely computed, this field is the
      * actual number of types in {@link #outputStack}.
      */
@@ -500,7 +500,7 @@ final class Frame {
 
     /**
      * Number of types that are initialized in the basic block.
-     * 
+     *
      * @see #initializations
      */
     private int initializationCount;
@@ -522,7 +522,7 @@ final class Frame {
 
     /**
      * Returns the output frame local variable type at the given index.
-     * 
+     *
      * @param local
      *            the index of the local that must be returned.
      * @return the output frame local variable type at the given index.
@@ -545,7 +545,7 @@ final class Frame {
 
     /**
      * Sets the output frame local variable type at the given index.
-     * 
+     *
      * @param local
      *            the index of the local that must be set.
      * @param type
@@ -568,7 +568,7 @@ final class Frame {
 
     /**
      * Pushes a new type onto the output frame stack.
-     * 
+     *
      * @param type
      *            the type that must be pushed.
      */
@@ -594,7 +594,7 @@ final class Frame {
 
     /**
      * Pushes a new type onto the output frame stack.
-     * 
+     *
      * @param cw
      *            the ClassWriter to which this label belongs.
      * @param desc
@@ -614,7 +614,7 @@ final class Frame {
 
     /**
      * Returns the int encoding of the given type.
-     * 
+     *
      * @param cw
      *            the ClassWriter to which this label belongs.
      * @param desc
@@ -688,7 +688,7 @@ final class Frame {
 
     /**
      * Pops a type from the output frame stack and returns its value.
-     * 
+     *
      * @return the type that has been popped from the output frame stack.
      */
     private int pop() {
@@ -702,7 +702,7 @@ final class Frame {
 
     /**
      * Pops the given number of types from the output frame stack.
-     * 
+     *
      * @param elements
      *            the number of types that must be popped.
      */
@@ -720,7 +720,7 @@ final class Frame {
 
     /**
      * Pops a type from the output frame stack.
-     * 
+     *
      * @param desc
      *            the descriptor of the type to be popped. Can also be a method
      *            descriptor (in this case this method pops the types
@@ -740,7 +740,7 @@ final class Frame {
     /**
      * Adds a new type to the list of types on which a constructor is invoked in
      * the basic block.
-     * 
+     *
      * @param var
      *            a type on a which a constructor is invoked.
      */
@@ -762,7 +762,7 @@ final class Frame {
     /**
      * Replaces the given type with the appropriate type if it is one of the
      * types on which a constructor is invoked in the basic block.
-     * 
+     *
      * @param cw
      *            the ClassWriter to which this label belongs.
      * @param t
@@ -799,7 +799,7 @@ final class Frame {
     /**
      * Initializes the input frame of the first basic block from the method
      * descriptor.
-     * 
+     *
      * @param cw
      *            the ClassWriter to which this label belongs.
      * @param access
@@ -835,7 +835,7 @@ final class Frame {
 
     /**
      * Simulates the action of the given instruction on the output stack frame.
-     * 
+     *
      * @param opcode
      *            the opcode of the instruction.
      * @param arg
@@ -1272,7 +1272,7 @@ final class Frame {
      * Merges the input frame of the given basic block with the input and output
      * frames of this basic block. Returns <tt>true</tt> if the input frame of
      * the given label has been changed by this operation.
-     * 
+     *
      * @param cw
      *            the ClassWriter to which this label belongs.
      * @param frame
@@ -1380,7 +1380,7 @@ final class Frame {
      * Merges the type at the given index in the given type array with the given
      * type. Returns <tt>true</tt> if the type array has been modified by this
      * operation.
-     * 
+     *
      * @param cw
      *            the ClassWriter to which this label belongs.
      * @param t
@@ -1417,6 +1417,7 @@ final class Frame {
                 // if t is the NULL type, merge(u,t)=u, so there is no change
                 return false;
             } else if ((t & (DIM | BASE_KIND)) == (u & (DIM | BASE_KIND))) {
+                // if t and u have the same dimension and same base kind
                 if ((u & BASE_KIND) == OBJECT) {
                     // if t is also a reference type, and if u and t have the
                     // same dimension merge(u,t) = dim(t) | common parent of the
@@ -1425,13 +1426,21 @@ final class Frame {
                             | cw.getMergedType(t & BASE_VALUE, u & BASE_VALUE);
                 } else {
                     // if u and t are array types, but not with the same element
-                    // type, merge(u,t)=java/lang/Object
-                    v = OBJECT | cw.addType("java/lang/Object");
+                    // type, merge(u,t) = dim(u) - 1 | java/lang/Object
+                    int vdim = ELEMENT_OF + (u & DIM);
+                    v = vdim | OBJECT | cw.addType("java/lang/Object");
                 }
             } else if ((t & BASE_KIND) == OBJECT || (t & DIM) != 0) {
-                // if t is any other reference or array type,
-                // merge(u,t)=java/lang/Object
-                v = OBJECT | cw.addType("java/lang/Object");
+                // if t is any other reference or array type, the merged type
+                // is min(udim, tdim) | java/lang/Object, where udim is the
+                // array dimension of u, minus 1 if u is an array type with a
+                // primitive element type (and similarly for tdim).
+                int tdim = (((t & DIM) == 0 || (t & BASE_KIND) == OBJECT) ? 0
+                        : ELEMENT_OF) + (t & DIM);
+                int udim = (((u & DIM) == 0 || (u & BASE_KIND) == OBJECT) ? 0
+                        : ELEMENT_OF) + (u & DIM);
+                v = Math.min(tdim, udim) | OBJECT
+                        | cw.addType("java/lang/Object");
             } else {
                 // if t is any other type, merge(u,t)=TOP
                 v = TOP;
