@@ -49,7 +49,7 @@ package jersey.repackaged.org.objectweb.asm;
  * <tt>visitLocalVariable</tt>, <tt>visitLocalVariableAnnotation</tt> and
  * <tt>visitLineNumber</tt> methods must be called <i>after</i> the labels
  * passed as arguments have been visited.
- * 
+ *
  * @author Eric Bruneton
  */
 public abstract class MethodVisitor {
@@ -68,7 +68,7 @@ public abstract class MethodVisitor {
 
     /**
      * Constructs a new {@link MethodVisitor}.
-     * 
+     *
      * @param api
      *            the ASM API version implemented by this visitor. Must be one
      *            of {@link Opcodes#ASM4} or {@link Opcodes#ASM5}.
@@ -79,7 +79,7 @@ public abstract class MethodVisitor {
 
     /**
      * Constructs a new {@link MethodVisitor}.
-     * 
+     *
      * @param api
      *            the ASM API version implemented by this visitor. Must be one
      *            of {@link Opcodes#ASM4} or {@link Opcodes#ASM5}.
@@ -101,7 +101,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a parameter of this method.
-     * 
+     *
      * @param name
      *            parameter name or null if none is provided.
      * @param access
@@ -120,7 +120,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits the default value of this annotation interface method.
-     * 
+     *
      * @return a visitor to the visit the actual default value of this
      *         annotation interface method, or <tt>null</tt> if this visitor is
      *         not interested in visiting this default value. The 'name'
@@ -137,7 +137,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits an annotation of this method.
-     * 
+     *
      * @param desc
      *            the class descriptor of the annotation class.
      * @param visible
@@ -154,7 +154,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits an annotation on a type in the method signature.
-     * 
+     *
      * @param typeRef
      *            a reference to the annotated type. The sort of this type
      *            reference must be {@link TypeReference#METHOD_TYPE_PARAMETER
@@ -190,7 +190,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits an annotation of a parameter this method.
-     * 
+     *
      * @param parameter
      *            the parameter index.
      * @param desc
@@ -210,7 +210,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a non standard attribute of this method.
-     * 
+     *
      * @param attr
      *            an attribute.
      */
@@ -271,7 +271,7 @@ public abstract class MethodVisitor {
      * and access flags, is implicit and must not be visited. Also, it is
      * illegal to visit two or more frames for the same code location (i.e., at
      * least one instruction must be visited between two calls to visitFrame).
-     * 
+     *
      * @param type
      *            the type of this stack map frame. Must be
      *            {@link Opcodes#F_NEW} for expanded frames, or
@@ -317,7 +317,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a zero operand instruction.
-     * 
+     *
      * @param opcode
      *            the opcode of the instruction to be visited. This opcode is
      *            either NOP, ACONST_NULL, ICONST_M1, ICONST_0, ICONST_1,
@@ -343,7 +343,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits an instruction with a single int operand.
-     * 
+     *
      * @param opcode
      *            the opcode of the instruction to be visited. This opcode is
      *            either BIPUSH, SIPUSH or NEWARRAY.
@@ -368,7 +368,7 @@ public abstract class MethodVisitor {
     /**
      * Visits a local variable instruction. A local variable instruction is an
      * instruction that loads or stores the value of a local variable.
-     * 
+     *
      * @param opcode
      *            the opcode of the local variable instruction to be visited.
      *            This opcode is either ILOAD, LLOAD, FLOAD, DLOAD, ALOAD,
@@ -386,7 +386,7 @@ public abstract class MethodVisitor {
     /**
      * Visits a type instruction. A type instruction is an instruction that
      * takes the internal name of a class as parameter.
-     * 
+     *
      * @param opcode
      *            the opcode of the type instruction to be visited. This opcode
      *            is either NEW, ANEWARRAY, CHECKCAST or INSTANCEOF.
@@ -404,7 +404,7 @@ public abstract class MethodVisitor {
     /**
      * Visits a field instruction. A field instruction is an instruction that
      * loads or stores the value of a field of an object.
-     * 
+     *
      * @param opcode
      *            the opcode of the type instruction to be visited. This opcode
      *            is either GETSTATIC, PUTSTATIC, GETFIELD or PUTFIELD.
@@ -426,7 +426,7 @@ public abstract class MethodVisitor {
     /**
      * Visits a method instruction. A method instruction is an instruction that
      * invokes a method.
-     * 
+     *
      * @param opcode
      *            the opcode of the type instruction to be visited. This opcode
      *            is either INVOKEVIRTUAL, INVOKESPECIAL, INVOKESTATIC or
@@ -439,16 +439,55 @@ public abstract class MethodVisitor {
      * @param desc
      *            the method's descriptor (see {@link Type Type}).
      */
+    @Deprecated
     public void visitMethodInsn(int opcode, String owner, String name,
             String desc) {
+        if (api >= Opcodes.ASM5) {
+            boolean itf = opcode == Opcodes.INVOKEINTERFACE;
+            visitMethodInsn(opcode, owner, name, desc, itf);
+            return;
+        }
         if (mv != null) {
             mv.visitMethodInsn(opcode, owner, name, desc);
         }
     }
 
+    /**
+     * Visits a method instruction. A method instruction is an instruction that
+     * invokes a method.
+     *
+     * @param opcode
+     *            the opcode of the type instruction to be visited. This opcode
+     *            is either INVOKEVIRTUAL, INVOKESPECIAL, INVOKESTATIC or
+     *            INVOKEINTERFACE.
+     * @param owner
+     *            the internal name of the method's owner class (see
+     *            {@link Type#getInternalName() getInternalName}).
+     * @param name
+     *            the method's name.
+     * @param desc
+     *            the method's descriptor (see {@link Type Type}).
+     * @param itf
+     *            if the method's owner class is an interface.
+     */
+    public void visitMethodInsn(int opcode, String owner, String name,
+            String desc, boolean itf) {
+        if (api < Opcodes.ASM5) {
+            if (itf != (opcode == Opcodes.INVOKEINTERFACE)) {
+                throw new IllegalArgumentException(
+                        "INVOKESPECIAL/STATIC on interfaces require ASM 5");
+            }
+            visitMethodInsn(opcode, owner, name, desc);
+            return;
+        }
+        if (mv != null) {
+            mv.visitMethodInsn(opcode, owner, name, desc, itf);
+        }
+    }
+
     /**
      * Visits an invokedynamic instruction.
-     * 
+     *
      * @param name
      *            the method's name.
      * @param desc
@@ -472,7 +511,7 @@ public abstract class MethodVisitor {
     /**
      * Visits a jump instruction. A jump instruction is an instruction that may
      * jump to another instruction.
-     * 
+     *
      * @param opcode
      *            the opcode of the type instruction to be visited. This opcode
      *            is either IFEQ, IFNE, IFLT, IFGE, IFGT, IFLE, IF_ICMPEQ,
@@ -492,7 +531,7 @@ public abstract class MethodVisitor {
     /**
      * Visits a label. A label designates the instruction that will be visited
      * just after it.
-     * 
+     *
      * @param label
      *            a {@link Label Label} object.
      */
@@ -511,7 +550,7 @@ public abstract class MethodVisitor {
      * future versions of the Java Virtual Machine. To easily detect new
      * constant types, implementations of this method should check for
      * unexpected constant types, like this:
-     * 
+     *
      * <pre>
      * if (cst instanceof Integer) {
      *     // ...
@@ -540,7 +579,7 @@ public abstract class MethodVisitor {
      *     // throw an exception
      * }
      * </pre>
-     * 
+     *
      * @param cst
      *            the constant to be loaded on the stack. This parameter must be
      *            a non null {@link Integer}, a {@link Float}, a {@link Long}, a
@@ -558,7 +597,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits an IINC instruction.
-     * 
+     *
      * @param var
      *            index of the local variable to be incremented.
      * @param increment
@@ -572,7 +611,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a TABLESWITCH instruction.
-     * 
+     *
      * @param min
      *            the minimum key value.
      * @param max
@@ -592,7 +631,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a LOOKUPSWITCH instruction.
-     * 
+     *
      * @param dflt
      *            beginning of the default handler block.
      * @param keys
@@ -609,7 +648,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a MULTIANEWARRAY instruction.
-     * 
+     *
      * @param desc
      *            an array type descriptor (see {@link Type Type}).
      * @param dims
@@ -625,7 +664,7 @@ public abstract class MethodVisitor {
      * Visits an annotation on an instruction. This method must be called just
      * <i>after</i> the annotated instruction. It can be called several times
      * for the same instruction.
-     * 
+     *
      * @param typeRef
      *            a reference to the annotated type. The sort of this type
      *            reference must be {@link TypeReference#INSTANCEOF INSTANCEOF},
@@ -669,7 +708,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a try catch block.
-     * 
+     *
      * @param start
      *            beginning of the exception handler's scope (inclusive).
      * @param end
@@ -696,7 +735,7 @@ public abstract class MethodVisitor {
      * called <i>after</i> the {@link #visitTryCatchBlock} for the annotated
      * exception handler. It can be called several times for the same exception
      * handler.
-     * 
+     *
      * @param typeRef
      *            a reference to the annotated type. The sort of this type
      *            reference must be {@link TypeReference#EXCEPTION_PARAMETER
@@ -725,7 +764,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a local variable declaration.
-     * 
+     *
      * @param name
      *            the name of a local variable.
      * @param desc
@@ -755,7 +794,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits an annotation on a local variable type.
-     * 
+     *
      * @param typeRef
      *            a reference to the annotated type. The sort of this type
      *            reference must be {@link TypeReference#LOCAL_VARIABLE
@@ -797,7 +836,7 @@ public abstract class MethodVisitor {
 
     /**
      * Visits a line number declaration.
-     * 
+     *
      * @param line
      *            a line number. This number refers to the source file from
      *            which the class was compiled.
@@ -816,7 +855,7 @@ public abstract class MethodVisitor {
     /**
      * Visits the maximum stack size and the maximum number of local variables
      * of the method.
-     * 
+     *
      * @param maxStack
      *            maximum stack size of the method.
      * @param maxLocals
